from django.utils.functional import cached_property
from embed_video.templatetags.embed_video_tags import VideoNode
from wagtail.core.blocks import ChooserBlock


class EmbedVideoChooserBlock(ChooserBlock):
    @cached_property
    def target_model(self):
        from wagtail_embed_videos import get_embed_video_model

        return get_embed_video_model()

    @cached_property
    def widget(self):
        from wagtail_embed_videos.widgets import AdminEmbedVideoChooser

        return AdminEmbedVideoChooser()

    def get_form_state(self, value):
        value_data = self.widget.get_value_data(value)
        if value_data is None:
            return None
        else:
            return {
                "id": value_data["id"],
                "edit_link": value_data["edit_url"],
                "title": value_data["title"],
                "preview": value_data["preview"],
            }

    def render_basic(self, value, context=None):
        if value:
            return VideoNode.embed(value.url, size="medium")
        else:
            return ""

    # TODO: implement get_comparison_class

    class Meta:
        icon = "media"



# Generated by Django 2.2.24 on 2021-11-03 07:46

from django.db import migrations


def add_embedvideo_permissions_to_admin_groups(apps, schema_editor):
    ContentType = apps.get_model("contenttypes.ContentType")
    Permission = apps.get_model("auth.Permission")
    Group = apps.get_model("auth.Group")

    # Get embed video permissions
    embedvideo_content_type, _created = ContentType.objects.get_or_create(
        model="embedvideo", app_label="wagtail_embed_videos"
    )

    add_embedvideo_permission, _created = Permission.objects.get_or_create(
        content_type=embedvideo_content_type, codename="add_embedvideo", defaults={"name": "Can add embed video"}
    )
    change_embedvideo_permission, _created = Permission.objects.get_or_create(
        content_type=embedvideo_content_type, codename="change_embedvideo", defaults={"name": "Can change embed video"}
    )
    delete_embedvideo_permission, _created = Permission.objects.get_or_create(
        content_type=embedvideo_content_type, codename="delete_embedvideo", defaults={"name": "Can delete embed video"}
    )

    # Assign it to Editors and Moderators groups
    for group in Group.objects.filter(name__in=["Editors", "Moderators"]):
        group.permissions.add(add_embedvideo_permission, change_embedvideo_permission, delete_embedvideo_permission)


def remove_embedvideo_permissions(apps, schema_editor):
    """Reverse the above additions of permissions."""
    ContentType = apps.get_model("contenttypes.ContentType")
    Permission = apps.get_model("auth.Permission")
    embedvideo_content_type = ContentType.objects.get(
        model="embedvideo",
        app_label="wagtail_embed_videos",
    )
    # This cascades to Group
    Permission.objects.filter(
        content_type=embedvideo_content_type, codename__in=("add_embedvideo", "change_embedvideo", "delete_embedvideo")
    ).delete()


class Migration(migrations.Migration):

    dependencies = [
        ("wagtail_embed_videos", "0003_embedvideo_collection"),
    ]

    operations = [
        migrations.RunPython(add_embedvideo_permissions_to_admin_groups, remove_embedvideo_permissions),
    ]



from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext

from wagtail_embed_videos.views.bulk_actions.embed_video_bulk_action import EmbedVideoBulkAction


class DeleteBulkAction(EmbedVideoBulkAction):
    display_name = _("Delete")
    action_type = "delete"
    aria_label = _("Delete selected embed videos")
    template_name = "wagtail_embed_videos/bulk_actions/confirm_bulk_delete.html"
    action_priority = 100
    classes = {'serious'}

    def check_perm(self, document):
        return self.permission_policy.user_has_permission_for_instance(self.request.user, 'delete', document)

    @classmethod
    def execute_action(cls, objects, **kwargs):
        num_parent_objects = len(objects)
        cls.get_default_model().objects.filter(pk__in=[obj.pk for obj in objects]).delete()
        return num_parent_objects, 0

    def get_success_message(self, num_parent_objects, num_child_objects):
        return ngettext(
            "%(num_parent_objects)d embed video has been deleted",
            "%(num_parent_objects)d embed videos have been deleted",
            num_parent_objects
        ) % {
            'num_parent_objects': num_parent_objects
        }



import sys

from django.conf import settings
from django.test import TestCase
from mock_django.models import ModelMock
from wagtail.images import get_image_model, get_image_model_string
from wagtail_embed_videos.models import EmbedVideo, create_thumbnail


class EmbedVideoTestCase(TestCase):
    def test_default_image_model(self):
        """Image model is the default Wagtail model"""
        # Remove module from cache
        # (https://docs.python.org/3/reference/import.html#the-module-cache)
        try:
            del sys.modules["wagtail_embed_videos.models"]
        except KeyError:
            pass  # Nothing to do

        self.assertEqual(get_image_model_string(), "wagtailimages.Image")

    def test_custom_image_model(self):
        """Image model is a custom model"""
        with self.settings(WAGTAILIMAGES_IMAGE_MODEL="testapp.CustomImage"):
            try:
                del sys.modules["wagtail_embed_videos.models"]
            except KeyError:
                pass  # Nothing to do

            self.assertEqual(get_image_model_string(), "testapp.CustomImage")

    def test_create_thumbnail(self):
        """Fetch a thumbnail from a video service."""
        video = EmbedVideo.objects.create(url="https://www.youtube.com/watch?v=dQw4w9WgXcQ", title="Click Me!")

        self.assertEqual("Click Me!", video.thumbnail.title)
        self.assertIsInstance(video.thumbnail, get_image_model())



from wagtail.admin.views.bulk_action import BulkAction
from wagtail_embed_videos import get_embed_video_model
from wagtail_embed_videos.permissions import permission_policy as embed_videos_permission_policy


class EmbedVideoBulkAction(BulkAction):
    permission_policy = embed_videos_permission_policy
    models = [get_embed_video_model()]

    def get_all_objects_in_listing_query(self, parent_id):
        listing_objects = self.model.objects.all()
        if parent_id is not None:
            listing_objects = listing_objects.filter(collection_id=parent_id)

        listing_objects = listing_objects.values_list('pk', flat=True)

        if 'q' in self.request.GET:
            query_string = self.request.GET.get('q', '')
            listing_objects = listing_objects.search(query_string).results()

        return listing_objects

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['items_with_no_access'] = [
            {
                'item': embed_video,
                'can_edit': self.permission_policy.user_has_permission_for_instance(self.request.user, 'change', embed_video)
            } for embed_video in context['items_with_no_access']
        ]
        return context



from wagtail.admin.edit_handlers import BaseChooserPanel

from .widgets import AdminEmbedVideoChooser


class EmbedVideoChooserPanel(BaseChooserPanel):
    object_type_name = "embed_video"

    def widget_overrides(self):
        return {self.field_name: AdminEmbedVideoChooser}


# TODO: EmbedVideoFieldComparison



# Generated by Django 2.2.24 on 2021-11-04 05:57

import django.db.models.deletion
import taggit.managers
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtail_embed_videos", "0005_auto_20211103_0802"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="embedvideo",
            options={"verbose_name": "embed video", "verbose_name_plural": "embed videos"},
        ),
        migrations.AlterField(
            model_name="embedvideo",
            name="created_at",
            field=models.DateTimeField(auto_now_add=True, db_index=True, verbose_name="created at"),
        ),
        migrations.AlterField(
            model_name="embedvideo",
            name="tags",
            field=taggit.managers.TaggableManager(
                blank=True, help_text=None, through="taggit.TaggedItem", to="taggit.Tag", verbose_name="tags"
            ),
        ),
        migrations.AlterField(
            model_name="embedvideo",
            name="thumbnail",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="wagtailimages.Image",
                verbose_name="thumbnail",
            ),
        ),
        migrations.AlterField(
            model_name="embedvideo",
            name="title",
            field=models.CharField(max_length=255, verbose_name="title"),
        ),
        migrations.AlterField(
            model_name="embedvideo",
            name="uploaded_by_user",
            field=models.ForeignKey(
                blank=True,
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to=settings.AUTH_USER_MODEL,
                verbose_name="uploaded by user",
            ),
        ),
    ]



from django.conf import settings
from django.core.exceptions import PermissionDenied
from django.core.paginator import Paginator
from django.shortcuts import get_object_or_404, redirect
from django.template.response import TemplateResponse
from django.urls import reverse
from django.utils.decorators import method_decorator
from django.utils.translation import gettext as _
from django.views.generic import TemplateView
from wagtail.admin import messages
from wagtail.admin.auth import PermissionPolicyChecker
from wagtail.admin.forms.search import SearchForm
from wagtail.admin.models import popular_tags_for_model
from wagtail.core.models import Collection
from wagtail.search import index as search_index

from wagtail_embed_videos import get_embed_video_model
from wagtail_embed_videos.forms import get_embed_video_form
from wagtail_embed_videos.permissions import permission_policy

permission_checker = PermissionPolicyChecker(permission_policy)

INDEX_PAGE_SIZE = getattr(settings, "WAGTAILEMBEDVIDEOS_INDEX_PAGE_SIZE", 20)
USAGE_PAGE_SIZE = getattr(settings, "WAGTAILEMBEDVIDEOS_USAGE_PAGE_SIZE", 20)


class BaseListingView(TemplateView):
    @method_decorator(permission_checker.require_any("add", "change", "delete"))
    def get(self, request):
        return super().get(request)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        # Get embed videos (filtered by user permission)
        embed_videos = (
            permission_policy.instances_user_has_any_permission_for(self.request.user, ["change", "delete"])
            .order_by("-created_at")
            .select_related("collection")
        )

        # Search
        query_string = None
        if "q" in self.request.GET:
            self.form = SearchForm(self.request.GET, placeholder=_("Search embed videos"))
            if self.form.is_valid():
                query_string = self.form.cleaned_data["q"]

                embed_videos = embed_videos.search(query_string)
        else:
            self.form = SearchForm(placeholder=_("Search embed videos"))

        # Filter by collection
        self.current_collection = None
        collection_id = self.request.GET.get("collection_id")
        if collection_id:
            try:
                self.current_collection = Collection.objects.get(id=collection_id)
                embed_videos = embed_videos.filter(collection=self.current_collection)
            except (ValueError, Collection.DoesNotExist):
                pass

        # Filter by tag
        self.current_tag = self.request.GET.get("tag")
        if self.current_tag:
            try:
                embed_videos = embed_videos.filter(tags__name=self.current_tag)
            except (AttributeError):
                self.current_tag = None

        paginator = Paginator(embed_videos, per_page=INDEX_PAGE_SIZE)
        embed_videos = paginator.get_page(self.request.GET.get("p"))

        context.update(
            {
                "embed_videos": embed_videos,
                "query_string": query_string,
                "is_searching": bool(query_string),
            }
        )

        return context


class IndexView(BaseListingView):
    template_name = "wagtail_embed_videos/embed_videos/index.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        collections = permission_policy.collections_user_has_any_permission_for(self.request.user, ["add", "change"])
        if len(collections) < 2:
            collections = None

        EmbedVideo = get_embed_video_model()

        context.update(
            {
                "search_form": self.form,
                "popular_tags": popular_tags_for_model(get_embed_video_model()),
                "current_tag": self.current_tag,
                "collections": collections,
                "current_collection": self.current_collection,
                "user_can_add": permission_policy.user_has_permission(self.request.user, "add"),
                'app_label': EmbedVideo._meta.app_label,
                'model_name': EmbedVideo._meta.model_name,
            }
        )
        return context


class ListingResultsView(BaseListingView):
    template_name = "wagtail_embed_videos/embed_videos/results.html"


@permission_checker.require("change")
def edit(request, embed_video_id):
    EmbedVideo = get_embed_video_model()
    EmbedVideoForm = get_embed_video_form(EmbedVideo)

    embed_video = get_object_or_404(EmbedVideo, id=embed_video_id)

    if not permission_policy.user_has_permission_for_instance(request.user, "change", embed_video):
        raise PermissionDenied

    if request.method == "POST":
        form = EmbedVideoForm(request.POST, request.FILES, instance=embed_video, user=request.user)
        if form.is_valid():
            form.save()

            # Reindex the embed video to make sure all tags are indexed
            search_index.insert_or_update_object(embed_video)

            messages.success(
                request,
                _("Video '{0}' updated.").format(embed_video.title),
                buttons=[
                    messages.button(reverse("wagtail_embed_videos:edit", args=(embed_video.id,)), _("Edit again"))
                ],
            )
            return redirect("wagtail_embed_videos:index")
        else:
            messages.error(request, _("The video could not be saved due to errors."))
    else:
        form = EmbedVideoForm(instance=embed_video, user=request.user)

    return TemplateResponse(
        request,
        "wagtail_embed_videos/embed_videos/edit.html",
        {
            "embed_video": embed_video,
            "form": form,
            "user_can_delete": permission_policy.user_has_permission_for_instance(request.user, "delete", embed_video),
        },
    )


@permission_checker.require("delete")
def delete(request, embed_video_id):
    embed_video = get_object_or_404(get_embed_video_model(), id=embed_video_id)

    if not permission_policy.user_has_permission_for_instance(request.user, "delete", embed_video):
        raise PermissionDenied

    if request.method == "POST":
        embed_video.delete()
        messages.success(request, _("Video '{0}' deleted.").format(embed_video.title))
        return redirect("wagtail_embed_videos:index")

    return TemplateResponse(
        request,
        "wagtail_embed_videos/embed_videos/confirm_delete.html",
        {
            "embed_video": embed_video,
        },
    )


@permission_checker.require("add")
def add(request):
    EmbedVideo = get_embed_video_model()
    EmbedVideoForm = get_embed_video_form(EmbedVideo)

    if request.method == "POST":
        embed_video = EmbedVideo(uploaded_by_user=request.user)
        form = EmbedVideoForm(request.POST, request.FILES, instance=embed_video, user=request.user)
        if form.is_valid():
            form.save()

            # Reindex the embed video to make sure all tags are indexed
            search_index.insert_or_update_object(embed_video)

            messages.success(
                request,
                _("Video '{0}' added.").format(embed_video.title),
                buttons=[messages.button(reverse("wagtail_embed_videos:edit", args=(embed_video.id,)), _("Edit"))],
            )
            return redirect("wagtail_embed_videos:index")
        else:
            messages.error(request, _("The video could not be created due to errors."))
    else:
        form = EmbedVideoForm(user=request.user)

    return TemplateResponse(
        request,
        "wagtail_embed_videos/embed_videos/add.html",
        {
            "form": form,
        },
    )


def usage(request, embed_video_id):
    embed_video = get_object_or_404(get_embed_video_model(), id=embed_video_id)

    paginator = Paginator(embed_video.get_usage(), per_page=USAGE_PAGE_SIZE)
    used_by = paginator.get_page(request.GET.get("p"))

    return TemplateResponse(
        request, "wagtail_embed_videos/embed_videos/usage.html", {"embed_video": embed_video, "used_by": used_by}
    )



from django.apps import AppConfig


class WagtailEmbedVideosAppConfig(AppConfig):
    name = "wagtail_embed_videos"
    label = "wagtail_embed_videos"
    verbose_name = "Wagtail embed videos"
    default_auto_field = "django.db.models.AutoField"



from types import SimpleNamespace

from django.conf import settings
from django.core.paginator import Paginator
from django.shortcuts import get_object_or_404
from django.template.loader import render_to_string
from django.template.response import TemplateResponse
from django.urls import reverse
from django.utils.translation import gettext as _
from django.views.generic.base import View
from embed_video.backends import detect_backend
from wagtail.admin.auth import PermissionPolicyChecker
from wagtail.admin.forms.search import SearchForm
from wagtail.admin.modal_workflow import render_modal_workflow
from wagtail.admin.models import popular_tags_for_model
from wagtail.core import hooks
from wagtail.search import index as search_index

from wagtail_embed_videos import get_embed_video_model
from wagtail_embed_videos.forms import get_embed_video_form
from wagtail_embed_videos.permissions import permission_policy

permission_checker = PermissionPolicyChecker(permission_policy)

CHOOSER_PAGE_SIZE = getattr(settings, "WAGTAILEMBEDVIDEOS_CHOOSER_PAGE_SIZE", 12)


def get_embed_video_result_data(embed_video):
    """
    helper function: given embed video, return the json data to pass back to the
    embed video chooser panel
    """
    if embed_video.thumbnail:
        preview = embed_video.thumbnail.get_rendition("max-165x165")
    else:
        preview = SimpleNamespace(
            url=detect_backend(embed_video.url).get_thumbnail_url(),
            width=165,
            height=92,
        )

    return {
        "id": embed_video.id,
        "edit_link": reverse("wagtail_embed_videos:edit", args=(embed_video.id,)),
        "title": embed_video.title,
        "preview": {
            "url": preview.url,
            "width": preview.width,
            "height": preview.height,
        },
    }


class BaseChooseView(View):
    def get(self, request):
        self.embed_video_model = get_embed_video_model()

        embed_videos = permission_policy.instances_user_has_any_permission_for(request.user, ["choose"]).order_by(
            "-created_at"
        )

        # allow hooks to modify the queryset
        for hook in hooks.get_hooks("construct_embed_video_chooser_queryset"):
            embed_videos = hook(embed_videos, request)

        collection_id = request.GET.get("collection_id")
        if collection_id:
            embed_videos = embed_videos.filter(collection=collection_id)

        self.is_searching = False
        self.q = None

        if 'q' in request.GET:
            self.search_form = SearchForm(request.GET)
            if self.search_form.is_valid():
                self.q = self.search_form.cleaned_data['q']
                self.is_searching = True
                embed_videos = embed_videos.search(self.q)
        else:
            self.search_form = SearchForm()

        if not self.is_searching:
            tag_name = request.GET.get("tag")
            if tag_name:
                embed_videos = embed_videos.filter(tags__name=tag_name)

        # Pagination
        paginator = Paginator(embed_videos, per_page=CHOOSER_PAGE_SIZE)
        self.embed_videos = paginator.get_page(request.GET.get('p'))
        return self.render_to_response()

    def get_context_data(self):
        return {
            'embed_videos': self.embed_videos,
            'is_searching': self.is_searching,
            'query_string': self.q,
        }

    def render_to_response(self):
        raise NotImplementedError()


class ChooseView(BaseChooseView):
    def get_context_data(self):
        context = super().get_context_data()

        if permission_policy.user_has_permission(self.request.user, 'add'):
            EmbedVideoForm = get_embed_video_form(self.embed_video_model)
            uploadform = EmbedVideoForm(user=self.request.user, prefix='embed_video-chooser-upload')
        else:
            uploadform = None

        collections = permission_policy.collections_user_has_permission_for(
            self.request.user, 'choose'
        )
        if len(collections) < 2:
            collections = None

        context.update({
            'searchform': self.search_form,
            'popular_tags': popular_tags_for_model(self.embed_video_model),
            'collections': collections,
            'uploadform': uploadform,
        })
        return context

    def render_to_response(self):
        return render_modal_workflow(
            self.request, 'wagtail_embed_videos/chooser/chooser.html', None, self.get_context_data(),
            json_data={
                'step': 'chooser',
                'error_label': _("Server Error"),
                'error_message': _("Report this error to your webmaster with the following information:"),
                'tag_autocomplete_url': reverse('wagtailadmin_tag_autocomplete'),
            }
        )


class ChooseResultsView(BaseChooseView):
    def render_to_response(self):
        return TemplateResponse(self.request, "wagtail_embed_videos/chooser/results.html", self.get_context_data())


def embed_video_chosen(request, embed_video_id):
    embed_video = get_object_or_404(get_embed_video_model(), id=embed_video_id)

    return render_modal_workflow(
        request,
        None,
        None,
        None,
        json_data={"step": "embed_video_chosen", "result": get_embed_video_result_data(embed_video)},
    )


@permission_checker.require("add")
def chooser_upload(request):
    EmbedVideo = get_embed_video_model()
    EmbedVideoForm = get_embed_video_form(EmbedVideo)

    if request.method == "POST":
        embed_video = EmbedVideo(uploaded_by_user=request.user)
        form = EmbedVideoForm(
            request.POST, request.FILES, instance=embed_video, user=request.user, prefix="embed_video-chooser-upload"
        )

        if form.is_valid():
            form.save()

            # Reindex the embed video to make sure all tags are indexed
            search_index.insert_or_update_object(embed_video)

            return render_modal_workflow(
                request,
                None,
                None,
                None,
                json_data={"step": "embed_video_chosen", "result": get_embed_video_result_data(embed_video)},
            )
    else:
        form = EmbedVideoForm(user=request.user, prefix="embed_video-chooser-upload")

    upload_form_html = render_to_string('wagtail_embed_videos/chooser/upload_form.html', {
        'form': form,
    }, request)

    return render_modal_workflow(
        request, None, None, None,
        json_data={
            'step': 'reshow_upload_form',
            'htmlFragment': upload_form_html
        }
    )



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import django.db.models.deletion
import embed_video.fields
import taggit.managers
import wagtail.search.index
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("taggit", "0002_auto_20150616_2121"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("wagtailimages", "0018_remove_rendition_filter"),
    ]

    operations = [
        migrations.CreateModel(
            name="EmbedVideo",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("title", models.CharField(max_length=255, verbose_name="Title")),
                ("url", embed_video.fields.EmbedVideoField()),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                (
                    "tags",
                    taggit.managers.TaggableManager(
                        blank=True, help_text=None, through="taggit.TaggedItem", to="taggit.Tag", verbose_name="Tags"
                    ),
                ),
                (
                    "thumbnail",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to="wagtailimages.Image",
                        verbose_name="Thumbnail",
                    ),
                ),
                (
                    "uploaded_by_user",
                    models.ForeignKey(
                        blank=True,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(wagtail.search.index.Indexed, models.Model),
        ),
    ]



# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

SECRET_KEY = "DUMMY_SECRET_KEY"

INTERNAL_IPS = []

# Application definition

PROJECT_APPS = ["wagtail_embed_videos.tests", "wagtail_embed_videos"]

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "taggit",
    "modelcluster",
    "wagtail.core",
    "wagtail.users",
    "wagtail.images",
    "wagtail.admin",
    "embed_video",
] + PROJECT_APPS

MIDDLEWARE = [
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "wagtail.core.middleware.SiteMiddleware",
    "wagtail.contrib.redirects.middleware.RedirectMiddleware",
]

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [os.path.join(BASE_DIR, "tests", "templates")],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
                "wagtail.tests.context_processors.do_not_use_static_url",
                "wagtail.contrib.settings.context_processors.settings",
            ]
        },
    }
]

ROOT_URLCONF = "wagtail_embed_videos.tests.urls"

# Database
# https://docs.djangoproject.com/en/1.8/ref/settings/#databases

DATABASES = {"default": {"ENGINE": "django.db.backends.sqlite3", "NAME": ":memory:"}}

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/2.1/howto/static-files/

# Absolute filesystem path to the directory that will hold user-uploaded files.
MEDIA_ROOT = os.path.join(BASE_DIR, "..", "media")

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash.
MEDIA_URL = "/media/"

STATIC_ROOT = os.path.join(BASE_DIR, "..", "staticfiles")

STATIC_URL = "/static/"

## Wagtail settings

WAGTAIL_SITE_NAME = "Test Site"



from django import forms
from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext

from wagtail_embed_videos.views.bulk_actions.embed_video_bulk_action import EmbedVideoBulkAction


class CollectionForm(forms.Form):
    def __init__(self, *args, **kwargs):
        user = kwargs.pop('user', None)
        super().__init__(*args, **kwargs)
        self.fields['collection'] = forms.ModelChoiceField(
            queryset=EmbedVideoBulkAction.permission_policy.collections_user_has_permission_for(user, 'add')
        )


class AddToCollectionBulkAction(EmbedVideoBulkAction):
    display_name = _("Add to collection")
    action_type = "add_to_collection"
    aria_label = _("Add selected embed videos to collection")
    template_name = "wagtail_embed_videos/bulk_actions/confirm_bulk_add_to_collection.html"
    action_priority = 30
    form_class = CollectionForm
    collection = None

    def check_perm(self, embed_video):
        return self.permission_policy.user_has_permission_for_instance(self.request.user, 'change', embed_video)

    def get_form_kwargs(self):
        return {
            **super().get_form_kwargs(),
            'user': self.request.user
        }

    def get_execution_context(self):
        return {
            'collection': self.cleaned_form.cleaned_data['collection']
        }

    @classmethod
    def execute_action(cls, embed_videos, collection=None, **kwargs):
        if collection is None:
            return
        num_parent_objects = cls.get_default_model().objects.filter(pk__in=[obj.pk for obj in embed_videos]).update(collection=collection)
        return num_parent_objects, 0

    def get_success_message(self, num_parent_objects, num_child_objects):
        collection = self.cleaned_form.cleaned_data['collection']
        return ngettext(
            "%(num_parent_objects)d embed video has been added to %(collection)s",
            "%(num_parent_objects)d embed videos have been added to %(collection)s",
            num_parent_objects
        ) % {
            'num_parent_objects': num_parent_objects,
            'collection': collection.name
        }



from django.conf import settings
from django.contrib import admin

from wagtail_embed_videos.models import EmbedVideo

if (
    hasattr(settings, "WAGTAILEMBEDVIDEOS_EMBEDVIDEO_MODEL")
    and settings.WAGTAILEMBEDVIDEOS_EMBEDVIDEO_MODEL != "wagtail_embed_videos.EmbedVideo"
):
    # This installation provides its own custom embed video class;
    # to avoid confusion, we won't expose the unused wagtail_embed_videos.EmbedVideo class
    # in the admin.
    pass
else:
    admin.site.register(EmbedVideo)



from .add_tags import AddTagsBulkAction
from .add_to_collection import AddToCollectionBulkAction
from .delete import DeleteBulkAction


__all__ = ['AddToCollectionBulkAction', 'AddTagsBulkAction', 'DeleteBulkAction']



import json
from types import SimpleNamespace

from django import forms
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from embed_video.backends import detect_backend
from wagtail.admin.staticfiles import versioned_static
from wagtail.admin.widgets import AdminChooser

from wagtail_embed_videos import get_embed_video_model


class AdminEmbedVideoChooser(AdminChooser):
    choose_one_text = _("Choose an embed video")
    choose_another_text = _("Choose another embed video")
    link_to_chosen_text = _("Edit this embed video")

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.embed_video_model = get_embed_video_model()

    def get_value_data(self, value):
        if value is None:
            return None
        elif isinstance(value, self.embed_video_model):
            embed_video = value
        else:  # assume embed video ID
            embed_video = self.embed_video_model.objects.get(pk=value)

        if embed_video.thumbnail:
            preview = embed_video.thumbnail.get_rendition("max-165x165")
        else:
            preview = SimpleNamespace(
                url=detect_backend(embed_video.url).get_thumbnail_url(),
                width=165,
                height=92,
            )

        return {
            "id": embed_video.pk,
            "title": embed_video.title,
            "preview": {
                "url": preview.url,
                "width": preview.width,
                "height": preview.height,
            },
            "edit_url": reverse("wagtail_embed_videos:edit", args=[embed_video.id]),
        }

    def render_html(self, name, value_data, attrs):
        value_data = value_data or {}
        original_field_html = super().render_html(name, value_data.get("id"), attrs)

        return render_to_string(
            "wagtail_embed_videos/widgets/embed_video_chooser.html",
            {
                "widget": self,
                "original_field_html": original_field_html,
                "attrs": attrs,
                "value": bool(value_data),  # only used by chooser.html to identify blank values
                "title": value_data.get("title", ""),
                "preview": value_data.get("preview", {}),
                "edit_url": value_data.get("edit_url", ""),
            },
        )

    def render_js_init(self, id_, name, value_data):
        return "createEmbedVideoChooser({0});".format(json.dumps(id_))

    @property
    def media(self):
        return forms.Media(
            js=[
                versioned_static("wagtail_embed_videos/js/embed-video-chooser-modal.js"),
                versioned_static("wagtail_embed_videos/js/embed-video-chooser.js"),
            ],
            css={
                "all": (versioned_static("wagtail_embed_videos/css/embed-video-chooser.css"),),
            },
        )



from wagtail.core.permission_policies.collections import CollectionOwnershipPermissionPolicy

from wagtail_embed_videos import get_embed_video_model
from wagtail_embed_videos.models import EmbedVideo

permission_policy = CollectionOwnershipPermissionPolicy(
    get_embed_video_model(), auth_model=EmbedVideo, owner_field_name="uploaded_by_user"
)



from django import forms
from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext

from wagtail.admin import widgets
from wagtail_embed_videos.views.bulk_actions.embed_video_bulk_action import EmbedVideoBulkAction


class TagForm(forms.Form):
    tags = forms.Field(widget=widgets.AdminTagWidget)


class AddTagsBulkAction(EmbedVideoBulkAction):
    display_name = _("Tag")
    action_type = "add_tags"
    aria_label = _("Add tags to the selected embed videos")
    template_name = "wagtail_embed_videos/bulk_actions/confirm_bulk_add_tags.html"
    action_priority = 20
    form_class = TagForm

    def check_perm(self, embed_video):
        return self.permission_policy.user_has_permission_for_instance(self.request.user, 'change', embed_video)

    def get_execution_context(self):
        return {
            'tags': self.cleaned_form.cleaned_data['tags'].split(',')
        }

    @classmethod
    def execute_action(cls, embed_videos, tags=[], **kwargs):
        num_parent_objects = 0
        if not tags:
            return
        for embed_video in embed_videos:
            num_parent_objects += 1
            embed_video.tags.add(*tags)
        return num_parent_objects, 0

    def get_success_message(self, num_parent_objects, num_child_objects):
        return ngettext(
            "New tags have been added to %(num_parent_objects)d embed video",
            "New tags have been added to %(num_parent_objects)d embed videos",
            num_parent_objects
        ) % {
            'num_parent_objects': num_parent_objects
        }



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

default_app_config = "wagtail_embed_videos.apps.WagtailEmbedVideosAppConfig"


def get_embed_video_model_string():
    """
    Get the dotted ``app.Model`` name for the embed video model as a string.
    Useful for developers making Wagtail plugins that need to refer to the
    embed video model, such as in foreign keys, but the model itself is not required.
    """
    return getattr(settings, "WAGTAILEMBEDVIDEOS_EMBEDVIDEO_MODEL", "wagtail_embed_videos.EmbedVideo")


def get_embed_video_model():
    """
    Get the embed video model from the ``WAGTAILEMBEDVIDEOS_EMBEDVIDEO_MODEL`` setting.
    Useful for developers making Wagtail plugins that need the embed video model.
    Defaults to the standard :class:`~wagtail_embed_videos.models.EmbedVideo` model
    if no custom model is defined.
    """
    from django.apps import apps

    model_string = get_embed_video_model_string()
    try:
        return apps.get_model(model_string, require_ready=False)
    except ValueError:
        raise ImproperlyConfigured("WAGTAILEMBEDVIDEOS_EMBEDVIDEO_MODEL must be of the form 'app_label.model_name'")
    except LookupError:
        raise ImproperlyConfigured(
            "WAGTAILEMBEDVIDEOS_EMBEDVIDEO_MODEL refers to model '%s' that has not been installed" % model_string
        )
